@@ -1,3 +1,4 @@
+import json
 import geojson
 from datasette import hookimpl
 from datasette.utils.asgi import Response
@@ -12,15 +13,22 @@ def register_output_renderer(datasette):
     }
 
 
-def render_geojson(datasette, columns, rows):
+async def render_geojson(datasette, columns, rows, request, database):
     if not can_render_geojson(datasette, columns):
         from datasette.views.base import DatasetteError
 
         raise DatasetteError("SQL query must include a geometry column")
 
-    features = [row_to_geojson(row) for row in rows]
-    fc = geojson.FeatureCollection(features=features)
+    db = datasette.get_database(database)
+    newline = bool(request.args.get("_nl", False))
+    features = [await row_to_geojson(row, db) for row in rows]
 
+    if newline:
+        # todo: stream this
+        lines = [json.dumps(feature) for feature in features]
+        return Response.text("\n".join(lines))
+
+    fc = geojson.FeatureCollection(features=features)
     return Response.json(dict(fc))
 
 
@@ -31,20 +39,27 @@ def can_render_geojson(datasette, columns):
     return "geometry" in columns
 
 
-def row_to_geojson(row):
+async def row_to_geojson(row, db):
     "Turn a row with a geometry column into a geojson feature"
     row = dict(row)
-    geometry = parse_geometry(row.pop("geometry"))
+    geometry = await parse_geometry(row.pop("geometry"), db)
 
     return geojson.Feature(geometry=geometry, properties=row)
 
 
-def parse_geometry(geometry):
+async def parse_geometry(geometry, db):
     "Start with a string, or binary blob, or dict, and return a geometry dict"
     if isinstance(geometry, dict):
         return geometry
 
-    if isinstance(geometry, (str, bytes)):
+    if isinstance(geometry, str):
         return geojson.loads(geometry)
 
-    raise ValueError("Unknown type")
+    if isinstance(geometry, bytes):
+        results = await db.execute(
+            "SELECT AsGeoJSON(:geometry)", {"geometry": geometry}
+        )
+
+        return geojson.loads(results.single_value())
+
+    raise ValueError(f"Unexpected geometry type: {type(geometry)}")
